<?php
namespace Ig\IgPayment\Services;

use PostFinanceCheckout\Sdk\ApiClient;
use PostFinanceCheckout\Sdk\Model\AddressCreate;
use PostFinanceCheckout\Sdk\Model\CustomerCreate;
use PostFinanceCheckout\Sdk\Model\TransactionCreate;
use TYPO3\CMS\Core\Context\Context;
use TYPO3\CMS\Core\SingletonInterface;
use TYPO3\CMS\Core\Site\Entity\Site;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManager;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;

class PostFinanceCheckoutService implements SingletonInterface
{

    protected array $settings = [];

    /**
     * Constructor
     */
    public function __construct()
    {
        $configurationManager = GeneralUtility::makeInstance(ConfigurationManager::class);
        $typoscript = $configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT);
        $settings = $typoscript['plugin.']['tx_igpayment.']['settings.'];
        $this->settings = ['pfCheckout' => $settings['pfCheckout.']];
    }

    public function getSettings(): array
    {
        return $this->settings;
    }
    

    public function getPaymentPageUrl(
        int $orderId,
        array $billingAddress,
        array $shippingAddress,
        array $lineItems,
        string $currency = 'CHF',
        string $successUrl = '',
        string $failUrl = ''
    ): string
    {
        // Get site language to get the payment page in the correct language
        $site = $GLOBALS['TYPO3_REQUEST']->getAttribute('site');
        $context = GeneralUtility::makeInstance(Context::class);
        $languageId = $context->getPropertyFromAspect('language', 'id');
        $language = $site->getLanguageById($languageId);

        // Configuration
        $spaceId = $this->settings['pfCheckout']['spaceId'];
        $userId = $this->settings['pfCheckout']['userId'];
        $secret = $this->settings['pfCheckout']['secret'];

        // Create transaction
        $transactionPayload = new TransactionCreate();
        $transactionPayload->setCurrency($currency)
                           ->setMetaData(['order_id' => $orderId])
                           ->setLineItems($lineItems)
                           ->setAutoConfirmationEnabled(true)
                           ->setLanguage($language->getLocale()->getLanguageCode());

        // Assign Billing Address
        if (!empty($billingAddress)) {
            $billingAddressObject = new AddressCreate($billingAddress);
            $transactionPayload->setBillingAddress($billingAddressObject);
        }

        // Assign Shipping Address
        if (!empty($shippingAddress)) {
            $shippingAddressObject = new AddressCreate($shippingAddress);
            $transactionPayload->setShippingAddress($shippingAddressObject);
        }

        // Assign URL to redirect to after successful payment
        if($successUrl) {
            $transactionPayload->setSuccessUrl($successUrl);
        }
        // Assign URL to redirect to after failed or aborted payment
        if ($failUrl) {
            $transactionPayload->setFailedUrl($failUrl);
        }

        // Setup API client
        $client = new ApiClient($userId, $secret);

        // Create the effective transaction
        $transaction = $client->getTransactionService()->create($spaceId, $transactionPayload);

        // Create Payment Page URL:
        $redirectionUrl = $client->getTransactionPaymentPageService()->paymentPageUrl($spaceId, $transaction->getId());

        return $redirectionUrl;
    }
}
